CON
  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000

  SHT_DATA      = 0 'brun                                   ' SHT-11 data pin
  SHT_CLOCK     = 1 'rød                                    ' SHT-11 clock pin
' SHT_GND           'grå
' SHT_VCC           'gul

  CLS         = $0                                      ' clear screen
  CR          = $D                                      ' carriage return
  LF          = $A
  Deg         = $B0                                     ' degree symbol

var
'    long pnt
    
    
OBJ

  term          : "serial_terminal"
  sht           : "Sensirion"
  f             : "Float32"
  fp            : "FloatString"

  ui            : "jm_lcd_ui"

PUB main | b, a

  a := 1
  term.start(12)                                        ' start terminal object
  f.start                                               ' start floating point object
  sht.start(SHT_DATA, SHT_CLOCK)                        ' start sensirion object

  ui.init                                                       ' initialize LCD UI
  ui.blctimer(5_000)                                           ' enable backlight
  ui.scrollstr(1, 1, 8, 250, @nuts)                             ' scroll line 1, r-to-l
  ui.rscrollstr(1, 2, 8, 250, @volts)                           ' scroll line 2, l-to-r
  repeat while ina[8]                                           ' hold while backlight on
  ui.cls                                                        ' clear the LCD

  repeat
    b := ui.getbtns                                             ' check buttons
    if b                                                        ' any pressed?
      ui.blctimer(10_000)                                        ' yes, restart backlight timer
      case b
        %0_00001..%0_10000 : ui.green                           ' joystick only?
        %1_00000           : ui.red                             ' Esc only?
        %1_00001..%1_10000 : ui.yellow                          ' combination?
    else
      ui.ledoff                                                 ' LED off if no buttons

    if b
      ui.dbputter(0)
    a -= 1
    if a == 0
      temp
      a := 25 * 2
    waitcnt(clkfreq / 25 + cnt)

PUB temp | i, rawTemp, rawHumidity, tempC, rh, Drh, pnt

  ' read SHT-11 sensor
    rawTemp := f.FFloat(sht.readTemperature)
    rawHumidity := f.FFloat(sht.readHumidity)
    tempC := celsius(rawTemp)
    rh := humidity(tempC, rawHumidity)
    Drh := dewpoint(tempC, rh)

    term.out($10)
    term.str(string(LF, "--------------------------------"))
    term.str(string(CR, " Temperatur    "))

    term.str(fp.FloatToFormat(tempC, 5, 1))
    term.str(string("  C", CR))
    term.str(string(CR, " Rel. Humidity "))

    term.str(fp.FloatToFormat(rh, 5, 1))
    term.str(string(" %RH"))
    term.str(string(CR, "-------------------------------"))

    term.str(string(CR, " Dewpoint      "))
    term.str(fp.FloatToFormat(Drh,5,1))
    term.str(string(CR, "--------------------------------",CR))
    
    'term.out($10)

    '--------------------------------------
    ui.moveto(1, 1) 
    ui.str(fp.FloatToFormat(tempC, 5, 1))
    ui.moveto(7, 1)
    ui.str(fp.FloatToFormat(Drh, 4,1))


    ui.moveto(1, 2)
    ui.str(fp.FloatToFormat(rh, 5, 1))
        
    ui.moveto(7, 2)
    pnt := fp.FloatToFormat(Drh, 4,1)
    ui.out(byte[pnt+2])
    ui.out(byte[pnt+3])


PUB celsius(t)
  ' from SHT1x/SHT7x datasheet using value for 3.5V supply
  ' celsius = -39.66 + (0.01 * t)
  return f.FAdd(-39.66, f.FMul(0.01, t))

PUB humidity(t, rh) | rhLinear
  ' rhLinear = -4.0 + (0.0405 * rh) + (-2.8e-6 * rh * rh)
  ' simplifies to: rhLinear = ((-2.8e-6 * rh) + 0.0405) * rh -4.0
  rhLinear := f.FAdd(f.FMul(f.FAdd(0.0405, f.FMul(-2.8e-6, rh)), rh), -4.0)
  ' rhTrue = (t - 25.0) * (0.01 + 0.00008 * rawRH) + rhLinear
  return f.FAdd(f.FMul(f.FSub(t, 25.0), f.FAdd(0.01, f.FMul(0.00008, rh))), rhLinear)

PUB dewpoint(t, rh) | h
  ' h = (log10(rh) - 2.0) / 0.4343 + (17.62 * t) / (243.12 + t)
  h := f.FAdd(f.FDiv(f.FSub(f.log10(rh), 2.0), 0.4343), f.FDiv(f.FMul(17.62, t), f.FAdd(243.12, t)))
  ' dewpoint = 243.12 * h / (17.62 - h)
  return f.FDiv(f.FMul(243.12, h), f.FSub(17.62, h))

dat

nuts            byte    "        NUTS    ", 0
volts           byte    "   VOLTS        ", 0


str             byte "1234  ",0
