'' =================================================================================================
''
''   File....... jm_debounce.spin
''   Purpose.... Debounces selected inputs
''   Author..... Jon "JonnyMac" McPhalen (aka Jon Williams)
''               Copyright (c) 2009 Jon McPhalen
''               -- see below for terms of use
''   E-mail..... jon@jonmcphalen.com
''   Started.... 05 JUL 2009
''   Updated.... 17 JUL 2009 
''
'' =================================================================================================


var

  long  cog

  long  dbMask                                                  ' bits to use
  long  dbBits                                                  ' debounced bits
  long  dbprel
  

pub init(mask, time, logic) | okay

'' Start debounce cog
'' -- mask defines the pins to scan (1 = scan, 0 = ignore) 
'' -- time is specified in milliseconds (10 to 100)
'' -- logic 1 = active high, logic 0 = active low

  finalize                                                      ' stop cog if running

'  if (time < 10) or (time > 100)                                ' verify debounce loop time (ms)
  if (time < 1) or (time > 100)                                ' verify debounce loop time (ms)
    time := 25                                                  ' default if out of range

  SCAN_DELAY := (clkfreq / 1_000) >> 3                          ' set one-shot cog parms       
  scanmask := mask
  scantime := time                                              
  scanmode := (logic > 0) & 1  
  scanbits := @dbBits

  dbMask := mask                                                ' set object parms
  dbBits := 0

  okay := cog := cognew(@debounce, 0) + 1                       ' start cog
   
  if okay
    dira := dira & !mask                                        ' make active pins inputs
  
  return okay


pub finalize

'' Stops debounce driver; frees a cog

  if cog
    cogstop(cog~ - 1)

    
pub active

'' Returns true if any of the selected inputs are active (1)

  return ((dbBits & dbMask) > 0)


pub getbits(mask)

'' Returns state of debounced bits
'' -- uses mask to select desired bit(s)

  return (dbBits & mask)


pub dbgetter

  return dbBits


pub dbputter(nn)

  dbBits := nn


pub dbaprel

  return dbprel

    
dat

                        org     0

debounce                andn    dira, scanmask                  ' make sure pins are inputs

'                        mov     dbtimer, SCAN_DELAY             ' initialize scan timer
'                        add     dbtimer, cnt                    ' sync with system counter
                      
newscan                 rdlong  scanresult, scanbits    wz
                if_nz   jmp     #newscan

                        mov     scanprel, #0
                        mov     dbtimer, SCAN_DELAY             ' initialize scan timer
                        add     dbtimer, cnt                    ' sync with system counter

               '         mov     scanresult, scanmask            ' initialize result
prel                    mov     scancount, scantime             ' load scan timing (ms)
                        shl     scancount, #3                   ' 8 scans per ms

scan                        test    scanmode, #1            wz      ' check mode                       
'scan           ' if_nz   and     scanresult, ina                 ' active-high scan
               ' if_z    andn    scanresult, ina                 ' active-low scan
               
                        mov     scanresult, ina
               if_nz    and     scanresult, scanmask
               if_z     andn    scanresult, scanmask
               
                        cmp     scanresult, scantemp    wz
                        mov     scantemp, scanresult
              if_nz     add     scanprel, #1          
              if_nz     jmp     #prel
              
                        waitcnt dbtimer, SCAN_DELAY             ' let timer expire, reload
                        djnz    scancount, #scan                ' update scans
                        
                        wrlong  scanresult, scanbits    wz            ' write result to hub
                        mov     scanadr, scanbits
                        add     scanadr, #4        
                        wrlong  scanprel, scanadr
                if_nz   mov     dbtimer, SCAN_TEMPO             ' initialize scan timer
                if_nz   add     dbtimer, cnt                    ' sync with system counter
                if_nz   waitcnt dbtimer, SCAN_TEMPO

             '           mov     scantempo, SCAN_TEMPO
             '           
again        '           mov     scanresult, ina         wz
             '           and     scanresult, scanresult    wz
             ' if_z      jmp     #j1
             '           djnz    scantempo, #again

                
j1                        jmp     #newscan                        ' start again 

' --------------------------------------------------------------------------------------------------

SCAN_DELAY              long    0-0                             ' counts between scans of ina
SCAN_TEMPO              long    20_000_000

scanmask                long    0-0                             ' mask of bits to scan
scantime                long    0-0                             ' scan window time, ms
scanmode                long    0-0                             ' 0 = active low, 1 = active high
scanbits                long    0-0                             ' hub address of scan result

scanadr                 res     1    
scancount               res     1                               ' loop control for scan
scanresult              res     1                               ' result of scan
scanprel                res     1
dbtimer                 res     1                               ' timer for scan window
scantemp                res     1
scantempo               res     1
                        fit     492
                        

dat

{{

  Copyright (c) 2009 Jon McPhalen (aka Jon Williams)

  Permission is hereby granted, free of charge, to any person obtaining a copy of this
  software and associated documentation files (the "Software"), to deal in the Software
  without restriction, including without limitation the rights to use, copy, modify,
  merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
  permit persons to whom the Software is furnished to do so, subject to the following
  conditions:

  The above copyright notice and this permission notice shall be included in all copies
  or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
  PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
  OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 

}}             