'' =================================================================================================
''
''   File....... jm_lcd_ui.spin
''   Purpose.... Unified object for LCD UI project (Nuts & Volts, September 2009)
''   Author..... Jon "JonnyMac" McPhalen (aka Jon Williams)
''               Copyright (c) 2009 Jon McPhalen
''               -- see below for terms of use
''   E-mail..... jon@jonmcphalen.com
''   Started.... 19 JUL 2009 
''   Updated.... 19 JUL 2009
''
'' =================================================================================================


con

  BTN_UP   = %000001                                            ' for LCD UI board 
  BTN_DN   = %000010 
  BTN_LF   = %000100 
  BTN_RT   = %001000 
  BTN_OK   = %010000 
  BTN_ESC  = %100000 

  BTN_MASK = BTN_UP | BTN_DN | BTN_RT | BTN_LF | BTN_OK | BTN_ESC


obj

  lcd  : "jm_lcd4_ez"
  blc  : "jm_oneshot"
  btns : "jm_debounce"
  led  : "jm_bicolor"


var


pub init | okay

'' Starts LCD UI object

  okay := lcd.init(8, 8, 2)                                     ' start lcd
  if okay
    okay := blc.init(8, 0, 0)                                   ' start backlight control one-shot
    if okay
'      okay := btns.init(BTN_MASK << 18, 50, 1)                  ' start button debouncing
      okay := btns.init(BTN_MASK << 18, 5, 1)                  ' start button debouncing
      if okay
        okay := led.init(17, 16)                                ' start bi-color LED


  if (okay == false)                                            ' if something failed
    lcd.finalize                                                ' remove all
    blc.finalize
    btns.finalize
    led.finalize

  return okay
  

pub cmd(c)

'' Write command byte to LCD

  lcd.cmd(c)
  

pub out(c)

'' Print character byte to LCD

  lcd.out(c)
 

pub str(pntr)

'' Print z-string at pntr

  lcd.str(pntr)     


pub dec(value) 

'' Print a decimal number

  lcd.dec(value)

 
pub hex(value, digits)

'' Print a hexadecimal number

  lcd.hex(value, digits)


pub bin(value, digits)

'' Print a binary number

  lcd.bin(value, digits)


pub in  

'' Reads byte at LCD cursor position

  return lcd.in


pub waitbusy  

'' Reads busy flag and cursor address
'' -- returns cursor address when busy flag has cleared

  return lcd.waitbusy 


pub setchar(c, pntr)

'' Write character map data to CGRAM
'' -- c is the custom character # (0..7)
'' -- pntr is the address of the bytes that define the cahracter

  return lcd.setchar(c, pntr) 


pub blon

'' Turn backlight control pin on

  lcd.blon


pub bloff

'' Turn backlight control pin off

  lcd.bloff


pub setbl(state)

'' Set backlight control pin to state

  lcd.setbl(state)


pub display(ison)

  lcd.display(ison)
  

pub cursor(mode) 

'' Sets LCD cursor style: off (0), underline (1), blinking bkg (2), uline+bkg (3)

  return cursor(mode) 


pub moveto(col, line) 

'' Moves DDRAM cursor to column, row position
'' -- home position is indexed as 1, 1

  lcd.moveto(col, line)


pub scrollstr(col, line, width, ms, pntr)

'' Scrolls string at pntr in window (width characters wide) at col/line
'' -- scroll direction is right-to-left
'' -- delay between character scrolls in milliseconds
'' -- scroll direction is right to left
'' -- strings should be padded with spaces for clean entry/exit

  lcd.scrollstr(col, line, width, ms, pntr) 
      

pub rscrollstr(col, line, width, ms, pntr)

'' Reverse scrolls string at pntr in window (width characters wide) at col/line
'' -- scroll direction is left-to-right
'' -- delay between character scrolls in milliseconds
'' -- scroll direction is right to left
'' -- strings should be padded with spaces for clean entry/exit

  lcd.rscrollstr(col, line, width, ms, pntr) 


pub cls

'' Clears LCD, moves cursor to 1, 1

  lcd.cmd(lcd#CLS)


pub blctimer(ms)

'' Runs backlight control timer
'' -- disables foreground control if active

  bloff
  blc.run(ms) 


pub dbgetter

  return btns.dbgetter


pub dbprel

  return  btns.dbaprel

  
pub dbputter(cn)

  btns.dbputter(cn)

  
pub hasbutton

'' Returns true if any of the selected inputs are active (1)

  return btns.active


pub getbtns

'' Returns state of debounced bits
'' -- uses mask to select desired bit(s)

  return ((btns.getbits(BTN_MASK << 18) >> 18) & BTN_MASK)


pub btnup

'' Returns true if UP joystick button is engaged

   return (btns.getbits(BTN_UP << 18) > 0) 


pub btndn

'' Returns true if DN joystick button is engaged

   return (btns.getbits(BTN_DN << 18) > 0)


pub btnlf

'' Returns true if LF joystick button is engaged

   return (btns.getbits(BTN_LF << 18) > 0)
   

pub btnrt

'' Returns true if UP joystick button is engaged

   return (btns.getbits(BTN_RT << 18) > 0)


pub btnok

'' Returns true if OK joystick button is engaged

   return (btns.getbits(BTN_OK << 18) > 0)


pub btnesc

'' Returns true if ESC button is engaged

   return (btns.getbits(BTN_ESC << 18) > 0)         


pub ledoff

  led.off


pub red

  led.red


pub green

  led.green


pub yellow

  led.yellow

  
dat

{{

  Copyright (c) 2009 Jon McPhalen (aka Jon Williams)

  Permission is hereby granted, free of charge, to any person obtaining a copy of this
  software and associated documentation files (the "Software"), to deal in the Software
  without restriction, including without limitation the rights to use, copy, modify,
  merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
  permit persons to whom the Software is furnished to do so, subject to the following
  conditions:

  The above copyright notice and this permission notice shall be included in all copies
  or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
  PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
  OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 

}}                   