'' =================================================================================================
''
''   File....... jm_oneshot.spin
''   Purpose.... "Background " one-shot pin control 
''   Author..... Jon "JonnyMac" McPhalen (aka Jon Williams)
''               Copyright (c) 2009 Jon McPhalen
''               -- see below for terms of use
''   E-mail..... jon@jonmcphalen.com
''   Started.... 18 JUL 2009
''   Updated.... 18 JUL 2009 
''
'' =================================================================================================


var

  long  cog
  
  long  osDuration                                              ' one-shot timing (milliseconds)


pub init(p, idle, ms) | okay

'' Starts one-shot object and activates pin (if ms > 0)
''    p is the pin that outputs the pulse
'' idle is the resting state of the pin (0 = low, 1 = high)
''   ms is the initial pulse duration

  finalize                                                      ' stop if already running

  if (p => 0) and (p =< 27)                                     ' valid? (protect rx, tx, i2c)
    MS_001 := clkfreq / 1_000                                   ' set one-shot cog parms
    osmask := 1 << p
    osidle := (idle > 0) & 1
    ospntr := @osDuration

    okay := cog := cognew(@oneshot, 0) + 1                      ' start cog
    run(ms)                                                     ' run initial pulse
       
  else
    okay := false 

  return okay 


pub finalize

'' Stops one-shot driver; frees a cog 

  if cog
    cogstop(cog~ - 1)


pub run(ms)

'' Runs one-shot pulse
'' -- timing specified in milliseconds
'' -- resets pulse timing if already running

  if (ms > 0)
    osDuration := ms
  else
    osDuration := 0


pub clear

'' Clears one-shot timing and output

  osDuration := 0


dat

                        org     0

oneshot                 test    osidle, #1              wc      ' check idle
                        muxc    outa, osmask                    ' set idle state
                        or      dira, osmask                    ' make pin an output
                        
                        mov     ms1timer, MS_001                ' load 1ms timer
                        add     ms1timer, cnt                   ' sync with system clock

osloop                  rdlong  ostimer, ospntr         wz      ' get one-shot timing
                if_z    muxc    outa, osmask                    ' if zero back to idle state
                if_nz   muxnc   outa, osmask                    ' else refresh active output
                if_nz   sub     ostimer, #1                     ' update the timiner
                        wrlong  ostimer, ospntr                 ' write it back to hub
                        waitcnt ms1timer, MS_001                ' wait 1ms
                        jmp     #osloop

' --------------------------------------------------------------------------------------------------

MS_001                  long    0-0                             ' ticks in 1ms

osmask                  long    0-0                             ' pin mask
osidle                  long    0-0                             ' pin idle state, 0 or 1
ospntr                  long    0-0                             ' pointer to os duration

ostimer                 res     1                               ' one-shot timing
ms1timer                res     1                               ' 1ms timing

                        fit     492
                        

dat

{{

  Copyright (c) 2009 Jon McPhalen (aka Jon Williams)

  Permission is hereby granted, free of charge, to any person obtaining a copy of this
  software and associated documentation files (the "Software"), to deal in the Software
  without restriction, including without limitation the rights to use, copy, modify,
  merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
  permit persons to whom the Software is furnished to do so, subject to the following
  conditions:

  The above copyright notice and this permission notice shall be included in all copies
  or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
  PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
  OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 

}}                   